# Chimeric mouse embryo (10X Genomics)

<script>
document.addEventListener("click", function (event) {
    if (event.target.classList.contains("rebook-collapse")) {
        event.target.classList.toggle("active");
        var content = event.target.nextElementSibling;
        if (content.style.display === "block") {
            content.style.display = "none";
        } else {
            content.style.display = "block";
        }
    }
})
</script>

<style>
.rebook-collapse {
  background-color: #eee;
  color: #444;
  cursor: pointer;
  padding: 18px;
  width: 100%;
  border: none;
  text-align: left;
  outline: none;
  font-size: 15px;
}

.rebook-content {
  padding: 0 18px;
  display: none;
  overflow: hidden;
  background-color: #f1f1f1;
}
</style>

## Introduction

This performs an analysis of the @pijuansala2019single dataset on mouse gastrulation.
Here, we examine chimeric embryos at the E8.5 stage of development 
where td-Tomato-positive embryonic stem cells (ESCs) were injected into a wild-type blastocyst.

## Data loading


```r
library(MouseGastrulationData)
sce.chimera <- WTChimeraData(samples=5:10)
sce.chimera
```

```
## class: SingleCellExperiment 
## dim: 29453 20935 
## metadata(0):
## assays(1): counts
## rownames(29453): ENSMUSG00000051951 ENSMUSG00000089699 ...
##   ENSMUSG00000095742 tomato-td
## rowData names(2): ENSEMBL SYMBOL
## colnames(20935): cell_9769 cell_9770 ... cell_30702 cell_30703
## colData names(11): cell barcode ... doub.density sizeFactor
## reducedDimNames(2): pca.corrected.E7.5 pca.corrected.E8.5
## mainExpName: NULL
## altExpNames(0):
```




```r
library(scater)
rownames(sce.chimera) <- uniquifyFeatureNames(
    rowData(sce.chimera)$ENSEMBL, rowData(sce.chimera)$SYMBOL)
```

## Quality control

Quality control on the cells has already been performed by the authors, so we will not repeat it here.
We additionally remove cells that are labelled as stripped nuclei or doublets.


```r
drop <- sce.chimera$celltype.mapped %in% c("stripped", "Doublet")
sce.chimera <- sce.chimera[,!drop]
```

## Normalization

We use the pre-computed size factors in `sce.chimera`.


```r
sce.chimera <- logNormCounts(sce.chimera)
```

## Variance modelling

We retain all genes with any positive biological component, to preserve as much signal as possible across a very heterogeneous dataset.


```r
library(scran)
dec.chimera <- modelGeneVar(sce.chimera, block=sce.chimera$sample)
chosen.hvgs <- dec.chimera$bio > 0
```


```r
par(mfrow=c(1,2))
blocked.stats <- dec.chimera$per.block
for (i in colnames(blocked.stats)) {
    current <- blocked.stats[[i]]
    plot(current$mean, current$total, main=i, pch=16, cex=0.5,
        xlab="Mean of log-expression", ylab="Variance of log-expression")
    curfit <- metadata(current)
    curve(curfit$trend(x), col='dodgerblue', add=TRUE, lwd=2)
}
```

<div class="figure">
<img src="pijuan-embryo_files/figure-html/unref-pijuan-var-1.png" alt="Per-gene variance as a function of the mean for the log-expression values in the Pijuan-Sala chimeric mouse embryo dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the variances." width="672" />
<p class="caption">(\#fig:unref-pijuan-var-1)Per-gene variance as a function of the mean for the log-expression values in the Pijuan-Sala chimeric mouse embryo dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the variances.</p>
</div><div class="figure">
<img src="pijuan-embryo_files/figure-html/unref-pijuan-var-2.png" alt="Per-gene variance as a function of the mean for the log-expression values in the Pijuan-Sala chimeric mouse embryo dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the variances." width="672" />
<p class="caption">(\#fig:unref-pijuan-var-2)Per-gene variance as a function of the mean for the log-expression values in the Pijuan-Sala chimeric mouse embryo dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the variances.</p>
</div><div class="figure">
<img src="pijuan-embryo_files/figure-html/unref-pijuan-var-3.png" alt="Per-gene variance as a function of the mean for the log-expression values in the Pijuan-Sala chimeric mouse embryo dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the variances." width="672" />
<p class="caption">(\#fig:unref-pijuan-var-3)Per-gene variance as a function of the mean for the log-expression values in the Pijuan-Sala chimeric mouse embryo dataset. Each point represents a gene (black) with the mean-variance trend (blue) fitted to the variances.</p>
</div>

## Merging

We use a hierarchical merge to first merge together replicates with the same genotype, 
and then merge samples across different genotypes.


```r
library(batchelor)
set.seed(01001001)
merged <- correctExperiments(sce.chimera, 
    batch=sce.chimera$sample, 
    subset.row=chosen.hvgs,
    PARAM=FastMnnParam(
        merge.order=list(
            list(1,3,5), # WT (3 replicates)
            list(2,4,6)  # td-Tomato (3 replicates)
        )
    )
)
```

We use the percentage of variance lost as a diagnostic:


```r
metadata(merged)$merge.info$lost.var
```

```
##              5         6         7         8        9       10
## [1,] 0.000e+00 0.0204433 0.000e+00 0.0169567 0.000000 0.000000
## [2,] 0.000e+00 0.0007389 0.000e+00 0.0004409 0.000000 0.015474
## [3,] 3.090e-02 0.0000000 2.012e-02 0.0000000 0.000000 0.000000
## [4,] 9.024e-05 0.0000000 8.272e-05 0.0000000 0.018047 0.000000
## [5,] 4.321e-03 0.0072518 4.124e-03 0.0078280 0.003831 0.007786
```

## Clustering


```r
g <- buildSNNGraph(merged, use.dimred="corrected")
clusters <- igraph::cluster_louvain(g)
colLabels(merged) <- factor(clusters$membership)
```

We examine the distribution of cells across clusters and samples.


```r
table(Cluster=colLabels(merged), Sample=merged$sample)
```

```
##        Sample
## Cluster   5   6   7   8   9  10
##      1  152  72  85  88 164 386
##      2   19   7  13  17  20  36
##      3  130  96 109  63 159 311
##      4   43  35  81  81  87 353
##      5   68  31 120 107  83 197
##      6  122  65  64  52  63 141
##      7  187 113 322 587 458 541
##      8   47  22  84  50  90 131
##      9  182  47 231 192 216 391
##      10  95  19  36  18  50  34
##      11 110  69  73  96 127 252
##      12   9   7  18  13  30  27
##      13   0   2   0  51   0   5
##      14  38  39  50  47 126 123
##      15  98  16 164 125 368 273
##      16 146  37 132 110 231 216
##      17 114  43  44  37  40 154
##      18  78  45 189 119 340 493
##      19  86  20  64  54 153  77
##      20 159  77 137 101 147 401
##      21   2   1   7   3  65 133
##      22  11  16  20   9  47  57
##      23   1   5   0  84   0  66
##      24 170  47 282 173 426 542
##      25 109  23 117  55 271 285
##      26 122  72 298 572 296 776
```

## Dimensionality reduction

We use an external algorithm to compute nearest neighbors for greater speed.


```r
merged <- runTSNE(merged, dimred="corrected", external_neighbors=TRUE)
merged <- runUMAP(merged, dimred="corrected", external_neighbors=TRUE)
```


```r
gridExtra::grid.arrange(
    plotTSNE(merged, colour_by="label", text_by="label", text_col="red"),
    plotTSNE(merged, colour_by="batch")
)
```

<div class="figure">
<img src="pijuan-embryo_files/figure-html/unref-pijuan-tsne-1.png" alt="Obligatory $t$-SNE plots of the Pijuan-Sala chimeric mouse embryo dataset, where each point represents a cell and is colored according to the assigned cluster (top) or sample of origin (bottom)." width="672" />
<p class="caption">(\#fig:unref-pijuan-tsne)Obligatory $t$-SNE plots of the Pijuan-Sala chimeric mouse embryo dataset, where each point represents a cell and is colored according to the assigned cluster (top) or sample of origin (bottom).</p>
</div>

## Session Info {-}

<button class="rebook-collapse">View session info</button>
<div class="rebook-content">
```
R version 4.1.0 beta (2021-05-03 r80259)
Platform: x86_64-pc-linux-gnu (64-bit)
Running under: Ubuntu 20.04.2 LTS

Matrix products: default
BLAS:   /home/biocbuild/bbs-3.14-bioc/R/lib/libRblas.so
LAPACK: /home/biocbuild/bbs-3.14-bioc/R/lib/libRlapack.so

locale:
 [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
 [3] LC_TIME=en_GB              LC_COLLATE=C              
 [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
 [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
 [9] LC_ADDRESS=C               LC_TELEPHONE=C            
[11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       

attached base packages:
[1] parallel  stats4    stats     graphics  grDevices utils     datasets 
[8] methods   base     

other attached packages:
 [1] batchelor_1.9.0             scran_1.21.1               
 [3] scater_1.21.0               ggplot2_3.3.3              
 [5] scuttle_1.3.0               MouseGastrulationData_1.7.0
 [7] SpatialExperiment_1.3.0     SingleCellExperiment_1.15.1
 [9] SummarizedExperiment_1.23.0 Biobase_2.53.0             
[11] GenomicRanges_1.45.0        GenomeInfoDb_1.29.0        
[13] IRanges_2.27.0              S4Vectors_0.31.0           
[15] BiocGenerics_0.39.0         MatrixGenerics_1.5.0       
[17] matrixStats_0.58.0          BiocStyle_2.21.0           
[19] rebook_1.3.0               

loaded via a namespace (and not attached):
  [1] AnnotationHub_3.1.0           BiocFileCache_2.1.0          
  [3] igraph_1.2.6                  BiocParallel_1.27.0          
  [5] digest_0.6.27                 BumpyMatrix_1.1.0            
  [7] htmltools_0.5.1.1             viridis_0.6.1                
  [9] magick_2.7.2                  fansi_0.4.2                  
 [11] magrittr_2.0.1                memoise_2.0.0                
 [13] ScaledMatrix_1.1.0            cluster_2.1.2                
 [15] limma_3.49.0                  Biostrings_2.61.0            
 [17] R.utils_2.10.1                colorspace_2.0-1             
 [19] blob_1.2.1                    rappdirs_0.3.3               
 [21] xfun_0.23                     dplyr_1.0.6                  
 [23] crayon_1.4.1                  RCurl_1.98-1.3               
 [25] jsonlite_1.7.2                graph_1.71.0                 
 [27] glue_1.4.2                    gtable_0.3.0                 
 [29] zlibbioc_1.39.0               XVector_0.33.0               
 [31] DelayedArray_0.19.0           BiocSingular_1.9.0           
 [33] DropletUtils_1.13.0           Rhdf5lib_1.15.0              
 [35] HDF5Array_1.21.0              scales_1.1.1                 
 [37] DBI_1.1.1                     edgeR_3.35.0                 
 [39] Rcpp_1.0.6                    viridisLite_0.4.0            
 [41] xtable_1.8-4                  dqrng_0.3.0                  
 [43] bit_4.0.4                     rsvd_1.0.5                   
 [45] ResidualMatrix_1.3.0          metapod_1.1.0                
 [47] httr_1.4.2                    dir.expiry_1.1.0             
 [49] ellipsis_0.3.2                farver_2.1.0                 
 [51] pkgconfig_2.0.3               XML_3.99-0.6                 
 [53] R.methodsS3_1.8.1             uwot_0.1.10                  
 [55] CodeDepends_0.6.5             sass_0.4.0                   
 [57] dbplyr_2.1.1                  locfit_1.5-9.4               
 [59] utf8_1.2.1                    labeling_0.4.2               
 [61] tidyselect_1.1.1              rlang_0.4.11                 
 [63] later_1.2.0                   AnnotationDbi_1.55.0         
 [65] munsell_0.5.0                 BiocVersion_3.14.0           
 [67] tools_4.1.0                   cachem_1.0.5                 
 [69] generics_0.1.0                RSQLite_2.2.7                
 [71] ExperimentHub_2.1.0           evaluate_0.14                
 [73] stringr_1.4.0                 fastmap_1.1.0                
 [75] yaml_2.2.1                    knitr_1.33                   
 [77] bit64_4.0.5                   purrr_0.3.4                  
 [79] KEGGREST_1.33.0               sparseMatrixStats_1.5.0      
 [81] mime_0.10                     R.oo_1.24.0                  
 [83] compiler_4.1.0                beeswarm_0.3.1               
 [85] filelock_1.0.2                curl_4.3.1                   
 [87] png_0.1-7                     interactiveDisplayBase_1.31.0
 [89] statmod_1.4.36                tibble_3.1.2                 
 [91] bslib_0.2.5.1                 stringi_1.6.2                
 [93] highr_0.9                     lattice_0.20-44              
 [95] bluster_1.3.0                 Matrix_1.3-3                 
 [97] vctrs_0.3.8                   pillar_1.6.1                 
 [99] lifecycle_1.0.0               rhdf5filters_1.5.0           
[101] BiocManager_1.30.15           jquerylib_0.1.4              
[103] BiocNeighbors_1.11.0          cowplot_1.1.1                
[105] bitops_1.0-7                  irlba_2.3.3                  
[107] httpuv_1.6.1                  R6_2.5.0                     
[109] bookdown_0.22                 promises_1.2.0.1             
[111] gridExtra_2.3                 vipor_0.4.5                  
[113] codetools_0.2-18              assertthat_0.2.1             
[115] rhdf5_2.37.0                  rjson_0.2.20                 
[117] withr_2.4.2                   GenomeInfoDbData_1.2.6       
[119] grid_4.1.0                    beachmat_2.9.0               
[121] rmarkdown_2.8                 DelayedMatrixStats_1.15.0    
[123] Rtsne_0.15                    shiny_1.6.0                  
[125] ggbeeswarm_0.6.0             
```
</div>
